import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_authentication/senior_authentication.dart';
import 'package:senior_authentication/src/screens/forgot_password/bloc/forgot_password_form_state.dart';

part 'forgot_password_event.dart';
part 'forgot_password_state.dart';

class ForgotPasswordBloc
    extends Bloc<ForgotPasswordEvent, ForgotPasswordState> {
  ForgotPasswordBloc({
    required this.authenticationRepository,
    required this.username,
  }) : super(
          ForgotPasswordState(username: username),
        );

  final AuthenticationRepository authenticationRepository;
  final String username;

  @override
  Stream<ForgotPasswordState> mapEventToState(
    ForgotPasswordEvent event,
  ) async* {
    if (event is UsernameChangedEvent) {
      yield state.copyWith(
        username: event.username,
        formState: const ForgotPasswordFormInitial(),
      );
    }

    if (event is ResetPasswordEvent) {
      yield state.copyWith(
        formState: ForgotPasswordFormLoading(),
      );

      try {
        await authenticationRepository.resetPassword(
            state.username, event.captcha, event.baseUrl);

        yield state.copyWith(
          formState: ForgotPasswordFormSuccess(),
        );
      } on Exception catch (_) {
        yield state.copyWith(
          formState: ForgotPasswordFormError(
            Exception('Erro ao tentar recuperar a senha.'),
          ),
        );
      }
    }
  }
}
